@@ -288,6 +288,9 @@ def start_new_series(cmdargs: argparse.Namespace) -> None:
                 logger.debug('Unrecognized cover letter format, will use as-is')
                 cover = cmsg.body
 
+            # Escape lines starting with "#" so they don't get lost
+            cover = re.sub(r'^(#.*)$', r'>\1', cover, flags=re.M)
+
             cover = (f'{cmsg.subject}\n\n'
                      f'EDITME: Imported from f{msgid}\n'
                      f'        Please review before sending.\n\n') + cover
@@ -451,21 +454,23 @@ def start_new_series(cmdargs: argparse.Namespace) -> None:
         # create a default cover letter and store it where the strategy indicates
         cover = ('EDITME: cover title for %s' % seriesname,
                  '',
-                 '# Lines starting with # will be removed from the cover letter. You can use',
-                 '# them to add notes or reminders to yourself.',
+                 '# Describe the purpose of this series. The information you put here',
+                 '# will be used by the project maintainer to make a decision whether',
+                 '# your patches should be reviewed, and in what priority order. Please be',
+                 '# very detailed and link to any relevant discussions or sites that the',
+                 '# maintainer can review to better understand your proposed changes. If you',
+                 '# only have a single patch in your series, the contents of the cover',
+                 '# letter will be appended to the "under-the-cut" portion of the patch.',
                  '',
-                 'EDITME: describe the purpose of this series. The information you put here',
-                 'will be used by the project maintainer to make a decision whether your',
-                 'patches should be reviewed, and in what priority order. Please be very',
-                 'detailed and link to any relevant discussions or sites that the maintainer',
-                 'can review to better understand your proposed changes. If you only have a',
-                 'single patch in your series, the contents of the cover letter will be',
-                 'appended to the "under-the-cut" portion of the patch.',
+                 '# Lines starting with # will be removed from the cover letter. You can',
+                 '# use them to add notes or reminders to yourself. If you want to use',
+                 '# markdown headers in your cover letter, start the line with ">#".',
                  '',
                  '# You can add trailers to the cover letter. Any email addresses found in',
-                 '# these trailers will be added to the addresses specified/generated during',
-                 '# the b4 send stage. You can also run "b4 prep --auto-to-cc" to auto-populate',
-                 '# the To: and Cc: trailers based on the code being modified.',
+                 '# these trailers will be added to the addresses specified/generated',
+                 '# during the b4 send stage. You can also run "b4 prep --auto-to-cc" to',
+                 '# auto-populate the To: and Cc: trailers based on the code being',
+                 '# modified.',
                  '',
                  'Signed-off-by: %s <%s>' % (usercfg.get('name', ''), usercfg.get('email', '')),
                  '',
@@ -480,7 +485,7 @@ def start_new_series(cmdargs: argparse.Namespace) -> None:
         if revision is None:
             revision = 1
         prefixes = list()
-        if cmdargs.set_prefixes and len(prefixes[0].strip()):
+        if cmdargs.set_prefixes:
             prefixes = list(cmdargs.set_prefixes)
         else:
             config = b4.get_main_config()
@@ -491,12 +496,14 @@ def start_new_series(cmdargs: argparse.Namespace) -> None:
             'series': {
                 'revision': revision,
                 'change-id': changeid,
-                'base-branch': basebranch,
                 'prefixes': prefixes,
             },
         }
         if thread_msgid:
             tracking['series']['from-thread'] = thread_msgid
+        if strategy != 'commit':
+            # We only need the base-branch info when using strategies other than 'commit'
+            tracking['series']['base-branch'] = basebranch
 
     store_cover(cover, tracking, new=True)
     if cherry_range:
@@ -561,8 +568,11 @@ def load_cover(strip_comments: bool = False) -> Tuple[str, dict]:
     logger.debug('tracking data: %s', tracking)
     if strip_comments:
         cover = re.sub(r'^#.*$', '', cover, flags=re.M)
+        # Unescape markdown headers
+        cover = re.sub(r'^>(#.*)$', r'\1', cover, flags=re.M)
         while '\n\n\n' in cover:
             cover = cover.replace('\n\n\n', '\n\n')
+
     return cover.strip(), tracking
 
 
@@ -686,17 +696,21 @@ def edit_cover() -> None:
     corecfg = b4.get_config_from_git(r'core\..*', {'editor': os.environ.get('EDITOR', 'vi')})
     editor = corecfg.get('editor')
     logger.debug('editor=%s', editor)
-    # We give it a suffix .rst in hopes that editors autoload restructured-text rules
-    with tempfile.NamedTemporaryFile(suffix='.rst') as temp_cover:
-        temp_cover.write(cover.encode())
-        temp_cover.seek(0)
+    # Use COMMIT_EDITMSG name in hopes that editors autoload git commit rules
+    with tempfile.TemporaryDirectory(prefix='b4-') as temp_dir:
+        temp_fpath = os.path.join(temp_dir, 'COMMIT_EDITMSG')
+        with open(temp_fpath, 'xb') as temp_cover:
+            temp_cover.write(cover.encode())
+
         sp = shlex.shlex(editor, posix=True)
         sp.whitespace_split = True
-        cmdargs = list(sp) + [temp_cover.name]
+        cmdargs = list(sp) + [temp_fpath]
         logger.debug('Running %s' % ' '.join(cmdargs))
         sp = subprocess.Popen(cmdargs)
         sp.wait()
-        new_cover = temp_cover.read().decode(errors='replace').strip()
+
+        with open(temp_fpath, 'rb') as temp_cover:
+            new_cover = temp_cover.read().decode(errors='replace').strip()
 
     if new_cover == cover:
         logger.info('Cover letter unchanged.')
@@ -801,6 +815,8 @@ def update_trailers(cmdargs: argparse.Namespace) -> None:
                 break
             prevparent = parent
             prevcommit = commit
+        if prevcommit is None:
+            prevcommit = end
         start = f'{prevcommit}~1'
     else:
         logger.critical('CRITICAL: Please specify -F msgid to look up trailers from remote.')
@@ -821,7 +837,7 @@ def update_trailers(cmdargs: argparse.Namespace) -> None:
         if not msg:
             continue
         commit_map[commit] = msg
-        body = msg.get_payload(decode=True).decode()
+        body, charset = b4.LoreMessage.get_payload(msg)
         patchid = b4.LoreMessage.get_patch_id(body)
         ls = b4.LoreSubject(msg.get('subject'))
         by_subject[ls.subject] = commit
@@ -854,11 +870,13 @@ def update_trailers(cmdargs: argparse.Namespace) -> None:
         lser = bbox.get_series(sloppytrailers=cmdargs.sloppytrailers)
         mismatches = list(lser.trailer_mismatches)
         for lmsg in lser.patches[1:]:
-            addtrailers = list(lmsg.followup_trailers)
-            if lser.has_cover and len(lser.patches[0].followup_trailers):
+            addtrailers = list()
+            if lmsg.followup_trailers:
+                addtrailers += list(lmsg.followup_trailers)
+            if lser.has_cover and lser.patches[0].followup_trailers:
                 addtrailers += list(lser.patches[0].followup_trailers)
             if not addtrailers:
-                logger.debug('No follow-up trailers received to: %s', lmsg.subject)
+                logger.debug('No new follow-up trailers to add to: %s', lmsg.subject)
                 continue
             commit = None
             if lmsg.subject in by_subject:
@@ -871,7 +889,8 @@ def update_trailers(cmdargs: argparse.Namespace) -> None:
                 logger.debug('No match for %s', lmsg.full_subject)
                 continue
 
-            parts = b4.LoreMessage.get_body_parts(commit_map[commit].get_payload(decode=True).decode())
+            mbody, mcharset = b4.LoreMessage.get_payload(commit_map[commit])
+            parts = b4.LoreMessage.get_body_parts(mbody)
             for fltr in addtrailers:
                 if fltr not in parts[2]:
                     if commit not in updates:
@@ -935,10 +954,13 @@ def get_addresses_from_cmd(cmdargs: List[str], msgbytes: bytes) -> List[Tuple[st
 def get_series_details(start_commit: Optional[str] = None) -> Tuple[str, str, str, List[str], str, str]:
     if not start_commit:
         start_commit = get_series_start()
-    gitargs = ['rev-parse', f'{start_commit}~1']
-    lines = b4.git_get_command_lines(None, gitargs)
-    base_commit = lines[0]
     strategy = get_cover_strategy()
+    if strategy == 'commit':
+        gitargs = ['rev-parse', f'{start_commit}~1']
+        lines = b4.git_get_command_lines(None, gitargs)
+        base_commit = lines[0]
+    else:
+        base_commit = start_commit
     if strategy == 'tip-commit':
         cover_commit = find_cover_commit()
         endrange = b4.git_revparse_obj(f'{cover_commit}~1')
@@ -989,19 +1011,24 @@ def make_msgid_tpt(change_id: str, revision: str, domain: Optional[str] = None)
         if myemail:
             domain = re.sub(r'^[^@]*@', '', myemail)
         else:
-            # Use the hostname of the system
-            import platform
-            domain = platform.node()
+            # Just use "b4" for the domain name (it doesn't need to be anything real)
+            domain = 'b4'
 
     chunks = change_id.rsplit('-', maxsplit=1)
     stablepart = chunks[0]
+    # Replace the change-id origin date with current date
+    chunks = stablepart.split('-', maxsplit=1)
+    if len(chunks) == 2 and len(chunks[0]) == 8:
+        # If someone uses b4 in year 10000, look me up.
+        stablepart = '%s-%s' % (datetime.date.today().strftime('%Y%m%d'), chunks[1])
+
     # Message-IDs must not be predictable to avoid stuffing attacks
     randompart = uuid.uuid4().hex[:12]
     msgid_tpt = f'<{stablepart}-v{revision}-%s-{randompart}@{domain}>'
     return msgid_tpt
 
 
-def get_cover_dests(cbody: str, hide: bool = True) -> Tuple[List, List, str]:
+def get_cover_dests(cbody: str) -> Tuple[List, List, str]:
     htrs, cmsg, mtrs, basement, sig = b4.LoreMessage.get_body_parts(cbody)
     tos = list()
     ccs = list()
@@ -1012,8 +1039,7 @@ def get_cover_dests(cbody: str, hide: bool = True) -> Tuple[List, List, str]:
         elif mtr.lname == 'cc':
             ccs.append(mtr.addr)
             mtrs.remove(mtr)
-    if hide:
-        cbody = b4.LoreMessage.rebuild_message(htrs, cmsg, mtrs, basement, sig)
+    cbody = b4.LoreMessage.rebuild_message(htrs, cmsg, mtrs, basement, sig)
     return tos, ccs, cbody
 
 
@@ -1041,7 +1067,7 @@ def add_cover(csubject: b4.LoreSubject, msgid_tpt: str, patches: List[Tuple[str,
 
 def mixin_cover(cbody: str, patches: List[Tuple[str, email.message.Message]]) -> None:
     msg = patches[0][1]
-    pbody = msg.get_payload(decode=True).decode()
+    pbody, pcharset = b4.LoreMessage.get_payload(msg)
     pheaders, pmessage, ptrailers, pbasement, psignature = b4.LoreMessage.get_body_parts(pbody)
     cheaders, cmessage, ctrailers, cbasement, csignature = b4.LoreMessage.get_body_parts(cbody)
     nbparts = list()
@@ -1081,6 +1107,9 @@ def mixin_cover(cbody: str, patches: List[Tuple[str, email.message.Message]]) ->
 
     pbody = b4.LoreMessage.rebuild_message(pheaders, pmessage, ptrailers, newbasement, csignature)
     msg.set_payload(pbody, charset='utf-8')
+    # Check if the new body now has 8bit content and fix CTR
+    if msg.get('Content-Transfer-Encoding') != '8bit' and not pbody.isascii():
+        msg.replace_header('Content-Transfer-Encoding', '8bit')
 
 
 def get_cover_subject_body(cover: str) -> Tuple[b4.LoreSubject, str]:
@@ -1113,8 +1142,7 @@ def get_mailfrom() -> Tuple[str, str]:
     return usercfg.get('name'), usercfg.get('email')
 
 
-def get_prep_branch_as_patches(movefrom: bool = True, thread: bool = True,
-                               addtracking: bool = True, hide_cover_to_cc: bool = False
+def get_prep_branch_as_patches(movefrom: bool = True, thread: bool = True, addtracking: bool = True
                                ) -> Tuple[List, List, str, List[Tuple[str, email.message.Message]]]:
     cover, tracking = load_cover(strip_comments=True)
 
@@ -1176,10 +1204,10 @@ def get_prep_branch_as_patches(movefrom: bool = True, thread: bool = True,
     ztracking = gzip.compress(bytes(json.dumps(tracking), 'utf-8'))
     b64tracking = base64.b64encode(ztracking).decode()
     # A little trick for pretty wrapping
-    wrapped = textwrap.wrap('X-B4-Tracking: v=1; b=' + b64tracking, width=75)
-    thdata = ' '.join(wrapped).replace('X-B4-Tracking: ', '')
+    wrapped = textwrap.wrap('X-B4-Tracking: v=1; b=' + b64tracking, subsequent_indent=' ', width=75)
+    thdata = ''.join(wrapped).replace('X-B4-Tracking: ', '')
 
-    alltos, allccs, cbody = get_cover_dests(cover_letter, hide=hide_cover_to_cc)
+    alltos, allccs, cbody = get_cover_dests(cover_letter)
     if len(patches) == 1:
         mixin_cover(cbody, patches)
     else:
@@ -1188,12 +1216,15 @@ def get_prep_branch_as_patches(movefrom: bool = True, thread: bool = True,
     if addtracking:
         patches[0][1].add_header('X-B4-Tracking', thdata)
 
-    tag_msg = f'{csubject.full_subject}\n\n{cover_letter}'
+    header = csubject.full_subject
+    if prefixes:
+        header = '[' + ', '.join(prefixes) + f'] {header}'
+    tag_msg = f'{header}\n\n{cover_letter}'
+
     return alltos, allccs, tag_msg, patches
 
 
-def get_sent_tag_as_patches(tagname: str, revision: int, hide_cover_to_cc: bool = False
-                            ) -> Tuple[List, List, List[Tuple[str, email.message.Message]]]:
+def get_sent_tag_as_patches(tagname: str, revision: int) -> Tuple[List, List, List[Tuple[str, email.message.Message]]]:
     cover, base_commit, change_id = get_base_changeid_from_tag(tagname)
 
     csubject, cbody = get_cover_subject_body(cover)
@@ -1210,7 +1241,7 @@ def get_sent_tag_as_patches(tagname: str, revision: int, hide_cover_to_cc: bool
                                       seriests=seriests,
                                       mailfrom=mailfrom)
 
-    alltos, allccs, cbody = get_cover_dests(cbody, hide=hide_cover_to_cc)
+    alltos, allccs, cbody = get_cover_dests(cbody)
     if len(patches) == 1:
         mixin_cover(cbody, patches)
     else:
@@ -1251,21 +1282,24 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
     mybranch = b4.git_get_current_branch()
 
     config = b4.get_main_config()
-    if not cmdargs.hide_cover_to_cc and config.get('send-hide-cover-to-cc', '').lower() in {'yes', 'true', '1'}:
-        cmdargs.hide_cover_to_cc = True
 
     tag_msg = None
     cl_msgid = None
     if cmdargs.resend:
-        tagname, revision = get_sent_tagname(mybranch, SENT_TAG_PREFIX, cmdargs.resend)
+        if cmdargs.resend == 'latest':
+            cover, tracking = load_cover()
+            revstr = tracking['series']['revision'] - 1
+        else:
+            revstr = cmdargs.resend
+
+        tagname, revision = get_sent_tagname(mybranch, SENT_TAG_PREFIX, revstr)
 
         if revision is None:
-            logger.critical('Could not figure out revision from %s', cmdargs.resend)
+            logger.critical('Could not figure out revision from %s', revstr)
             sys.exit(1)
 
         try:
-            todests, ccdests, patches = get_sent_tag_as_patches(tagname, revision=revision,
-                                                                hide_cover_to_cc=cmdargs.hide_cover_to_cc)
+            todests, ccdests, patches = get_sent_tag_as_patches(tagname, revision=revision)
         except RuntimeError as ex:
             logger.critical('CRITICAL: Failed to convert tag to patches: %s', ex)
             sys.exit(1)
@@ -1273,21 +1307,27 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
         logger.info('Converted the tag to %s messages', len(patches))
 
     else:
-        # Check if the cover letter has 'EDITME' in it
         cover, tracking = load_cover(strip_comments=True)
-        if 'EDITME' in cover:
+        status = b4.git_get_repo_status()
+        if len(status):
+            logger.critical('CRITICAL: Repository contains uncommitted changes.')
+            logger.critical('          Stash or commit them first.')
+            sys.exit(1)
+
+        try:
+            todests, ccdests, tag_msg, patches = get_prep_branch_as_patches()
+        except RuntimeError as ex:
+            logger.critical('CRITICAL: Failed to convert range to patches: %s', ex)
+            sys.exit(1)
+
+        # Check if "EDITME" shows up in the first message
+        if b'EDITME' in b4.LoreMessage.get_msg_as_bytes(patches[0][1]):
             logger.critical('CRITICAL: Looks like the cover letter needs to be edited first.')
             logger.info('---')
             logger.info(cover)
             logger.info('---')
             sys.exit(1)
 
-        try:
-            todests, ccdests, tag_msg, patches = get_prep_branch_as_patches(
-                hide_cover_to_cc=cmdargs.hide_cover_to_cc)
-        except RuntimeError as ex:
-            logger.critical('CRITICAL: Failed to convert range to patches: %s', ex)
-            sys.exit(1)
         logger.info('Converted the branch to %s messages', len(patches))
 
     usercfg = b4.get_user_config()
@@ -1307,20 +1347,18 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
         for commit, msg in patches:
             if not msg:
                 continue
-            body = msg.get_payload(decode=True).decode()
+            body, charset = b4.LoreMessage.get_payload(msg)
             btrs, junk = b4.LoreMessage.find_trailers(body)
             for btr in btrs:
                 if btr.type != 'person':
                     continue
                 if btr.addr[1] in seen:
                     continue
-                if commit and btr.lname == 'cc':
-                    # CC's in individual patches don't get added to global Cc's,
-                    # we use the pcss dict to track them.
+                if commit:
                     if commit not in pccs:
                         pccs[commit] = list()
-                    pccs[commit].append(btr.addr)
-                    # Doesn't get added to seen, in case a later patch puts it into global
+                    if btr.addr not in pccs[commit]:
+                        pccs[commit].append(btr.addr)
                     continue
                 seen.add(btr.addr[1])
                 if btr.lname == 'to':
@@ -1381,15 +1419,20 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
         pathlib.Path(cmdargs.output_dir).mkdir(parents=True, exist_ok=True)
 
     sconfig = b4.get_sendemail_config()
-    endpoint = config.get('send-endpoint-web', '')
-    if not re.search(r'^https?://', endpoint):
-        endpoint = None
-    if not endpoint and not sconfig.get('smtpserver'):
-        # Use the default endpoint if we are in the kernel repo
-        topdir = b4.git_get_toplevel()
-        if os.path.exists(os.path.join(topdir, 'Kconfig')):
-            logger.debug('No sendemail configs found, will use the default web endpoint')
-            endpoint = DEFAULT_ENDPOINT
+    # If we have an smtp server defined, always use that instead of the endpoint
+    # we may make this configurable in the future, but this almost always makes sense
+    endpoint = None
+    if not sconfig.get('smtpserver'):
+        endpoint = config.get('send-endpoint-web', '')
+        if not re.search(r'^https?://', endpoint):
+            logger.debug('Endpoint does not start with https, ignoring: %s', endpoint)
+            endpoint = None
+        if not endpoint:
+            # Use the default endpoint if we are in the kernel repo
+            topdir = b4.git_get_toplevel()
+            if os.path.exists(os.path.join(topdir, 'Kconfig')):
+                logger.debug('No sendemail configs found, will use the default web endpoint')
+                endpoint = DEFAULT_ENDPOINT
 
     # Give the user the last opportunity to bail out
     if not cmdargs.dryrun:
@@ -1429,7 +1472,22 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
             logger.info('  - with envelope-from: %s', fromaddr)
 
             smtpserver = sconfig.get('smtpserver', 'localhost')
-            logger.info('  - via SMTP server %s', smtpserver)
+            if '/' in smtpserver:
+                logger.info('  - via local command %s', smtpserver)
+                if cmdargs.reflect and sconfig.get('b4-really-reflect-via') != smtpserver:
+                    logger.critical('---')
+                    logger.critical('CRITICAL: Cowardly refusing to reflect via %s.', smtpserver)
+                    logger.critical('          There is no guarantee that this command will do the right thing')
+                    logger.critical('          and will not send mail to actual addressees.')
+                    logger.critical('---')
+                    logger.critical('If you are ABSOLUTELY SURE that this command will do the right thing,')
+                    logger.critical('add the following to the [sendemail] section:')
+                    logger.critical('b4-really-reflect-via = %s', smtpserver)
+                    sys.exit(1)
+
+            else:
+                logger.info('  - via SMTP server %s', smtpserver)
+
         if not (cmdargs.reflect or cmdargs.resend):
             logger.info('  - tag and reroll the series to the next revision')
         logger.info('')
@@ -1474,10 +1532,12 @@ def cmd_send(cmdargs: argparse.Namespace) -> None:
                     mycc.append(pair)
         elif not commit and len(pccs):
             # the cover letter gets sent to folks with individual patch cc's
+            _seen = set(seen)
             for _commit, _ccs in pccs.items():
                 for pair in _ccs:
-                    if pair[1] not in seen:
+                    if pair[1] not in _seen:
                         mycc.append(pair)
+                        _seen.add(pair[1])
         if mycc and not myto:
             # Move all Cc's into To when there's no To:
             myto = mycc
@@ -1722,8 +1782,11 @@ def show_info() -> None:
     if 'history' in ts:
         for rn, links in reversed(ts['history'].items()):
             tagname, revision = get_sent_tagname(mybranch, SENT_TAG_PREFIX, rn)
-            cover, base_commit, change_id = get_base_changeid_from_tag(tagname)
-            print('series-%s: %s..%s %s' % (rn, base_commit[:12], tagname, links[0]))
+            try:
+                cover, base_commit, change_id = get_base_changeid_from_tag(tagname)
+                print('series-%s: %s..%s %s' % (rn, base_commit[:12], tagname, links[0]))
+            except RuntimeError:
+                logger.debug('No tag matching %s', tagname)
 
 
 def force_revision(forceto: int) -> None:
@@ -1816,32 +1879,19 @@ def auto_to_cc() -> None:
             logger.debug('added %s to seen', ltr.addr[1])
             extras.append(ltr)
 
-    tos, ccs, tag_msg, patches = get_prep_branch_as_patches()
+    try:
+        tos, ccs, tag_msg, patches = get_prep_branch_as_patches()
+    except RuntimeError:
+        logger.info('No commits in branch')
+        return
+
     logger.info('Collecting To/Cc addresses')
     # Go through the messages to make to/cc headers
     for commit, msg in patches:
-        if not msg:
-            continue
-        payload = msg.get_payload(decode=True).decode()
-        parts = b4.LoreMessage.get_body_parts(payload)
-        for ltr in parts[2]:
-            if ltr.lname == 'cc':
-                # We treat Cc: in individual patches differently from all other trailers:
-                # they only receive individual patches, not the entire series.
-                continue
-            if not ltr.addr:
-                continue
-            if ltr.addr[1] in seen:
-                continue
-            seen.add(ltr.addr[1])
-            logger.debug('added %s to seen', ltr.addr[1])
-            # Make it a Cc: trailer
-            ltr.name = 'Cc'
-            extras.append(ltr)
-
-        if not commit:
+        if not msg or not commit:
             continue
 
+        logger.debug('Collecting from: %s', msg.get('subject'))
         msgbytes = msg.as_bytes()
         for tname, pairs in (('To', get_addresses_from_cmd(tocmd, msgbytes)),
                              ('Cc', get_addresses_from_cmd(cccmd, msgbytes))):
@@ -1849,6 +1899,7 @@ def auto_to_cc() -> None:
                 if pair[1] not in seen:
                     seen.add(pair[1])
                     ltr = b4.LoreTrailer(name=tname, value=b4.format_addrs([pair]))
+                    logger.debug('  => %s', ltr.as_string())
                     extras.append(ltr)
 
     if not extras:
@@ -1890,12 +1941,6 @@ def cmd_prep(cmdargs: argparse.Namespace) -> None:
         logger.critical('          Stash or commit them first.')
         sys.exit(1)
 
-    if cmdargs.edit_cover:
-        return edit_cover()
-
-    if cmdargs.auto_to_cc:
-        return auto_to_cc()
-
     if cmdargs.reroll:
         msgid = cmdargs.reroll
         msgs = b4.get_pi_thread_by_msgid(msgid, onlymsgids={msgid}, nocache=True)
@@ -1905,7 +1950,7 @@ def cmd_prep(cmdargs: argparse.Namespace) -> None:
                 if b4.LoreMessage.get_clean_msgid(msg) == msgid:
                     # Prepare annotated tag body from the cover letter
                     lsubject = b4.LoreSubject(msg.get('subject'))
-                    cbody = msg.get_payload(decode=True).decode()
+                    cbody, charset = b4.LoreMessage.get_payload(msg)
                     prefixes = lsubject.get_extra_prefixes()
                     if prefixes:
                         subject = '[%s] %s' % (' '.join(prefixes), lsubject.subject)
@@ -1922,23 +1967,34 @@ def cmd_prep(cmdargs: argparse.Namespace) -> None:
     if cmdargs.show_info:
         return show_info()
 
-    if cmdargs.force_revision:
-        return force_revision(cmdargs.force_revision)
-
     if cmdargs.format_patch:
         return format_patch(cmdargs.format_patch)
 
     if cmdargs.compare_to:
         return compare(cmdargs.compare_to)
 
-    if cmdargs.set_prefixes:
-        return set_prefixes(cmdargs.set_prefixes)
-
-    if is_prep_branch():
-        logger.critical('CRITICAL: This appears to already be a b4-prep managed branch.')
+    if cmdargs.enroll_base and cmdargs.new_series_name:
+        logger.critical('CRITICAL: -n NEW_SERIES_NAME and -e [ENROLL_BASE] can not be used together.')
         sys.exit(1)
 
-    return start_new_series(cmdargs)
+    if cmdargs.enroll_base or cmdargs.new_series_name:
+        if is_prep_branch() and not cmdargs.fork_point:
+            logger.critical('CRITICAL: This appears to already be a b4-prep managed branch.')
+            sys.exit(1)
+
+        start_new_series(cmdargs)
+
+    if cmdargs.force_revision:
+        force_revision(cmdargs.force_revision)
+
+    if cmdargs.set_prefixes:
+        set_prefixes(cmdargs.set_prefixes)
+
+    if cmdargs.auto_to_cc:
+        auto_to_cc()
+
+    if cmdargs.edit_cover:
+        return edit_cover()
 
 
 def cmd_trailers(cmdargs: argparse.Namespace) -> None:
