@@ -75,10 +75,7 @@ def git_get_commit_message(gitdir, rev):
 
 
 def make_reply(reply_template, jsondata, gitdir):
-    body = Template(reply_template).safe_substitute(jsondata)
-    # Conform to email standards
-    body = body.replace('\n', '\r\n')
-    msg = email.message_from_string(body)
+    msg = email.message.EmailMessage()
     msg['From'] = '%s <%s>' % (jsondata['myname'], jsondata['myemail'])
     excludes = b4.get_excluded_addrs()
     newto = b4.cleanup_email_addrs([(jsondata['fromname'], jsondata['fromemail'])], excludes, gitdir)
@@ -110,6 +107,10 @@ def make_reply(reply_template, jsondata, gitdir):
     mydomain = jsondata['myemail'].split('@')[1]
     msg['Message-Id'] = email.utils.make_msgid(idstring='b4-ty', domain=mydomain)
     msg['Date'] = email.utils.formatdate(localtime=True)
+    body = Template(reply_template).safe_substitute(jsondata)
+    msg.set_payload(body, charset='utf-8')
+    msg.set_charset('utf-8')
+
     return msg
 
 
@@ -385,18 +386,19 @@ def send_messages(listing, branch, cmdargs):
     datadir = b4.get_data_dir()
     fromaddr = None
     smtp = None
-    if cmdargs.sendemail:
-        # See if we have sendemail-identity set
-        config = b4.get_main_config()
-        identity = config.get('sendemail-identity')
+    config = b4.get_main_config()
+
+    if cmdargs.sendemail or config.get('ty-send-email', 'no').lower() in ['yes', 'true', '1']:
+        send_email = True
         try:
-            smtp, fromaddr = b4.get_smtp(identity)
+            smtp, fromaddr = b4.get_smtp()
         except Exception as ex:  # noqa
             logger.critical('Failed to configure the smtp connection:')
             logger.critical(ex)
             sys.exit(1)
     else:
         # We write .thanks notes
+        send_email = False
         # Check if the outdir exists and if it has any .thanks files in it
         if not os.path.exists(cmdargs.outdir):
             os.mkdir(cmdargs.outdir)
@@ -425,14 +427,11 @@ def send_messages(listing, branch, cmdargs):
             msgids.append(pdata[2])
 
         outgoing += 1
-        msg.set_charset('utf-8')
-        msg.replace_header('Content-Transfer-Encoding', '8bit')
-        if cmdargs.sendemail:
+        if send_email:
             if not fromaddr:
                 fromaddr = jsondata['myemail']
-            logger.info('  Sending: %s', msg.get('subject'))
-            # We never want to use the web endpoint for this (it's only for submitting patches)
-            b4.send_mail(smtp, [msg], fromaddr, dryrun=cmdargs.dryrun, use_web_endpoint=False)
+            logger.info('  Sending: %s', b4.LoreMessage.clean_header(msg.get('subject')))
+            b4.send_mail(smtp, [msg], fromaddr, dryrun=cmdargs.dryrun)
         else:
             slug_from = re.sub(r'\W', '_', jsondata['fromemail'])
             slug_subj = re.sub(r'\W', '_', jsondata['subject'])
@@ -440,8 +439,8 @@ def send_messages(listing, branch, cmdargs):
             slug = re.sub(r'_+', '_', slug)
             outfile = os.path.join(cmdargs.outdir, '%s.thanks' % slug)
             logger.info('  Writing: %s', outfile)
-            with open(outfile, 'w') as fh:
-                fh.write(msg.as_string(policy=b4.emlpolicy))
+            with open(outfile, 'wb') as fh:
+                fh.write(msg.as_bytes(policy=b4.emlpolicy))
         if cmdargs.dryrun:
             logger.info('Dry run, preserving tracked series.')
         else:
@@ -459,7 +458,7 @@ def send_messages(listing, branch, cmdargs):
     if not pwstate:
         pwstate = config.get('pw-accept-state')
 
-    if cmdargs.sendemail:
+    if send_email:
         if cmdargs.dryrun:
             logger.info('DRYRUN: generated %s thank-you letters', outgoing)
         else:
@@ -628,7 +627,7 @@ def get_branch_info(gitdir, branch):
 
     BRANCH_INFO = dict()
 
-    remotecfg = b4.get_config_from_git('branch\\.%s\\.*' % branch)
+    remotecfg = b4.get_config_from_git('branch\\.%s\\..*' % branch)
     if remotecfg is None or 'remote' not in remotecfg:
         # Did not find a matching branch entry, so look at remotes
         gitargs = ['remote', 'show']
